@@ -84,6 +84,8 @@ bool eapol_calculate_mic(enum ie_rsn_akm_suite akm, const uint8_t *kck,
 		case IE_RSN_AKM_SUITE_SAE_SHA256:
 		case IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256:
 			return cmac_aes(kck, 16, frame, frame_len, mic, 16);
+		case IE_RSN_AKM_SUITE_OWE:
+			return hmac_sha256(kck, 16, frame, frame_len, mic, 16);
 		default:
 			return false;
 		}
@@ -128,6 +130,10 @@ bool eapol_verify_mic(enum ie_rsn_akm_suite akm, const uint8_t *kck,
 		case IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256:
 			checksum = l_checksum_new_cmac_aes(kck, 16);
 			break;
+		case IE_RSN_AKM_SUITE_OWE:
+			checksum = l_checksum_new_hmac(L_CHECKSUM_SHA256,
+								kck, 16);
+			break;
 		default:
 			return false;
 		}
@@ -165,12 +171,12 @@ uint8_t *eapol_decrypt_key_data(enum ie_rsn_akm_suite akm, const uint8_t *kek,
 		break;
 	case EAPOL_KEY_DESCRIPTOR_VERSION_AKM_DEFINED:
 		/*
-		 * TODO: for now, only SAE is supported under the AKM_DEFINED
-		 * key descriptor version. Once 8021x suites are added for this
-		 * type this will need to be expanded to handle the AKM types in
-		 * its own switch.
+		 * TODO: for now, only SAE/OWE (group 19) is supported under the
+		 * AKM_DEFINED key descriptor version. Once 8021x suites are
+		 * added for this type this will need to be expanded to handle
+		 * the AKM types in its own switch.
 		 */
-		if (!IE_AKM_IS_SAE(akm))
+		if (!IE_AKM_IS_SAE(akm) && akm != IE_RSN_AKM_SUITE_OWE)
 			return NULL;
 
 		/* Fall through */
@@ -564,8 +570,10 @@ static struct eapol_key *eapol_create_common(
 	out_frame->key_length = 0;
 	out_frame->key_replay_counter = L_CPU_TO_BE64(key_replay_counter);
 	memcpy(out_frame->key_nonce, snonce, sizeof(out_frame->key_nonce));
+
 	out_frame->key_data_len = L_CPU_TO_BE16(extra_len);
-	memcpy(out_frame->key_data, extra_data, extra_len);
+	if (extra_len)
+		memcpy(out_frame->key_data, extra_data, extra_len);
 
 	return out_frame;
 }
@@ -974,8 +982,10 @@ static void eapol_handle_ptk_1_of_4(struct eapol_sm *sm,
 	} else if (pmkid) {
 		uint8_t own_pmkid[16];
 
-		if (handshake_state_get_pmkid(sm->handshake, own_pmkid) &&
-				memcmp(pmkid, own_pmkid, 16)) {
+		if (!handshake_state_get_pmkid(sm->handshake, own_pmkid))
+			goto error_unspecified;
+
+		if (memcmp(pmkid, own_pmkid, 16)) {
 			l_debug("Authenticator sent a PMKID that didn't match");
 
 			/*
@@ -990,15 +1000,12 @@ static void eapol_handle_ptk_1_of_4(struct eapol_sm *sm,
 
 			/*
 			 * Some APs are known to send a PMKID KDE with all
-			 * zeros for the PMKID.  Likely we can still
+			 * zeros for the PMKID.  Others just send seemingly
+			 * random data.  Likely we can still
 			 * successfully negotiate a handshake, so ignore this
 			 * for now and treat it as if the PMKID KDE was not
 			 * included
 			 */
-			if (util_mem_is_zero(pmkid, 16))
-				l_debug("PMKID is all zero, ignoring");
-			else
-				goto error_unspecified;
 		}
 	}
 
@@ -2377,7 +2384,7 @@ void __eapol_set_config(struct l_settings *config)
 {
 	if (!l_settings_get_uint(config, "EAPoL",
 			"max_4way_handshake_time", &eapol_4way_handshake_time))
-		eapol_4way_handshake_time = 2;
+		eapol_4way_handshake_time = 5;
 }
 
 bool eapol_init()
